@@ -45,12 +45,59 @@ def test_main_warns_outside_project_root_with_selection(tmp_path, monkeypatch, c
         finally:
             run.apply_settings(original_settings)
 
-    assert exc.value.code == 1
+    assert str(exc.value).startswith("error: test path `tests/sample.tql` does not exist")
+    captured = capsys.readouterr()
+    assert captured.out.strip() == ""
+
+
+def test_main_accepts_satellite_selection_without_project_root(tmp_path, monkeypatch, capsys):
+    original_settings = run._settings
+    library_root = tmp_path / "library"
+    library_root.mkdir()
+    package_dir = library_root / "pkg"
+    package_dir.mkdir()
+    (package_dir / "package.yaml").write_text("name: pkg\nversion: 0.0.1\n")
+    (package_dir / "tests").mkdir()
+
+    monkeypatch.setenv("TENZIR_TEST_ROOT", str(library_root))
+    monkeypatch.delenv("TENZIR_BINARY", raising=False)
+    monkeypatch.delenv("TENZIR_NODE_BINARY", raising=False)
+    monkeypatch.chdir(library_root)
+
+    try:
+        run.main(["pkg"])
+    finally:
+        run.apply_settings(original_settings)
+
     captured = capsys.readouterr()
     lines = [line for line in captured.out.splitlines() if line]
-    assert lines[0].startswith(f"{run.INFO} no tenzir-test project detected")
-    assert lines[1] == f"{run.INFO} run from your project root or provide --root."
-    assert lines[2] == f"{run.INFO} Ignoring provided selection(s): tests/sample.tql"
+    assert lines == [f"{run.INFO} no tests selected"]
+
+
+def test_main_accepts_current_directory_selection_without_project_root(
+    tmp_path, monkeypatch, capsys
+):
+    original_settings = run._settings
+    library_root = tmp_path / "library"
+    library_root.mkdir()
+    package_dir = library_root / "pkg"
+    package_dir.mkdir()
+    (package_dir / "package.yaml").write_text("name: pkg\nversion: 0.0.1\n")
+    (package_dir / "tests").mkdir()
+
+    monkeypatch.setenv("TENZIR_TEST_ROOT", str(library_root))
+    monkeypatch.delenv("TENZIR_BINARY", raising=False)
+    monkeypatch.delenv("TENZIR_NODE_BINARY", raising=False)
+    monkeypatch.chdir(library_root)
+
+    try:
+        run.main(["."])
+    finally:
+        run.apply_settings(original_settings)
+
+    captured = capsys.readouterr()
+    lines = [line for line in captured.out.splitlines() if line]
+    assert lines == [f"{run.INFO} no tests selected"]
 
 
 def test_format_summary_reports_counts_and_percentages() -> None:
@@ -638,6 +685,42 @@ write_json
     assert summary.total == 1
 
 
+def test_count_queue_tests_includes_suite_members(tmp_path: Path) -> None:
+    class StubRunner(run.Runner):
+        def __init__(self) -> None:
+            super().__init__(name="stub")
+
+        def collect_tests(  # noqa: ARG002
+            self, path: Path
+        ) -> set[tuple[run.Runner, Path]]:
+            return set()
+
+        def purge(self) -> None:
+            return None
+
+        def run(self, test: Path, update: bool, coverage: bool = False) -> bool:  # noqa: ARG002
+            return True
+
+    runner = StubRunner()
+    solo_test = tmp_path / "solo.tql"
+    solo_test.write_text("version\n", encoding="utf-8")
+    suite_dir = tmp_path / "suite"
+    suite_dir.mkdir()
+    suite_info = run.SuiteInfo(name="demo", directory=suite_dir)
+    suite_tests: list[run.TestQueueItem] = []
+    for index in range(3):
+        test_path = suite_dir / f"case-{index}.tql"
+        test_path.write_text("version\n", encoding="utf-8")
+        suite_tests.append(run.TestQueueItem(runner=runner, path=test_path))
+
+    queue: list[run.RunnerQueueItem] = [
+        run.TestQueueItem(runner=runner, path=solo_test),
+        run.SuiteQueueItem(suite=suite_info, tests=suite_tests, fixtures=tuple()),
+    ]
+
+    assert run._count_queue_tests(queue) == 4
+
+
 def test_cli_rejects_partial_suite_selection(
     tmp_path: Path, capsys: pytest.CaptureFixture[str]
 ) -> None:
@@ -686,6 +769,8 @@ def test_cli_rejects_partial_suite_selection(
                 runner_summary=False,
                 fixture_summary=False,
                 show_summary=False,
+                show_diff_output=True,
+                show_diff_stat=True,
                 jobs=1,
                 keep_tmp_dirs=False,
                 passthrough=False,
@@ -710,6 +795,8 @@ def test_cli_rejects_partial_suite_selection(
                 runner_summary=False,
                 fixture_summary=False,
                 show_summary=False,
+                show_diff_output=True,
+                show_diff_stat=True,
                 jobs=1,
                 keep_tmp_dirs=False,
                 passthrough=False,
@@ -745,6 +832,82 @@ def test_detailed_summary_order(capsys):
     assert skipped_index < failed_index < table_start
 
 
+def test_print_diff_default_layout(capsys):
+    original_show_diff = run.should_show_diff_output()
+    original_show_stat = run.should_show_diff_stat()
+    try:
+        run.set_show_diff_output(True)
+        run.set_show_diff_stat(True)
+        run.print_diff(b"line\nbeta\n", b"line\ngamma\n", Path("tests/example.txt"))
+    finally:
+        run.set_show_diff_output(original_show_diff)
+        run.set_show_diff_stat(original_show_stat)
+
+    output = capsys.readouterr().out.splitlines()
+    assert output[0].startswith(f"{run._BLOCK_INDENT}┌ tests/example.txt")
+    expected_counter = run._format_diff_counter(1, 1)
+    assert "\033[32m1(+)\033[0m/\033[31m1(-)\033[0m" in output[0]
+    assert output[0].endswith(expected_counter)
+    assert output[1] == f"{run._BLOCK_INDENT}│ @@ -1,2 +1,2 @@"
+    assert output[2] == f"{run._BLOCK_INDENT}│  line"
+    assert output[3].startswith(f"{run._BLOCK_INDENT}│ \033[31m-beta")
+    assert output[4].startswith(f"{run._BLOCK_INDENT}│ \033[32m+gamma")
+    assert output[5] == f"{run._BLOCK_INDENT}└ 2 lines changed"
+
+
+def test_print_diff_no_diff_outputs_stat_only(capsys):
+    original_show_diff = run.should_show_diff_output()
+    original_show_stat = run.should_show_diff_stat()
+    try:
+        run.set_show_diff_output(False)
+        run.set_show_diff_stat(True)
+        run.print_diff(b"line\nbeta\n", b"line\ngamma\n", Path("tests/example.txt"))
+    finally:
+        run.set_show_diff_output(original_show_diff)
+        run.set_show_diff_stat(original_show_stat)
+
+    output = capsys.readouterr().out.splitlines()
+    expected_header = (
+        f"{run._BLOCK_INDENT}┌ tests/example.txt "
+        f"\033[32m1(+)\033[0m/\033[31m1(-)\033[0m {run._format_diff_counter(1, 1)}"
+    )
+    assert output == [
+        expected_header,
+        f"{run._BLOCK_INDENT}└ 2 lines changed",
+    ]
+
+
+def test_print_diff_stat_disabled_shows_only_diff(capsys):
+    original_show_diff = run.should_show_diff_output()
+    original_show_stat = run.should_show_diff_stat()
+    try:
+        run.set_show_diff_output(True)
+        run.set_show_diff_stat(False)
+        run.print_diff(b"line\nbeta\n", b"line\ngamma\n", Path("tests/example.txt"))
+    finally:
+        run.set_show_diff_output(original_show_diff)
+        run.set_show_diff_stat(original_show_stat)
+
+    output = capsys.readouterr().out.splitlines()
+    assert output[0] == f"{run._BLOCK_INDENT}┌ tests/example.txt"
+    assert output[1] == f"{run._BLOCK_INDENT}│ @@ -1,2 +1,2 @@"
+    assert output[-1] == f"{run._BLOCK_INDENT}└ 2 lines changed"
+
+
+def test_print_diff_disabled_outputs_nothing(capsys):
+    original_show_diff = run.should_show_diff_output()
+    original_show_stat = run.should_show_diff_stat()
+    try:
+        run.set_show_diff_output(False)
+        run.set_show_diff_stat(False)
+        run.print_diff(b"line\nbeta\n", b"line\ngamma\n", Path("tests/example.txt"))
+    finally:
+        run.set_show_diff_output(original_show_diff)
+        run.set_show_diff_stat(original_show_stat)
+
+    assert capsys.readouterr().out.strip() == ""
+
+
 def test_describe_project_root_detects_standard_project(tmp_path: Path) -> None:
     project_root = tmp_path / "proj"
     tests_dir = project_root / "tests"
@@ -1008,22 +1171,126 @@ def test_print_execution_plan_lists_projects(capsys):
     run._print_execution_plan(plan, display_base=root)
 
     output = capsys.readouterr().out
-    assert "executing 2 projects" in output
-    assert "■ root-project" in output
-    assert "□ satellite-project" in output
+    assert "found 2 projects" in output
+    assert f"{run.INFO}   ■ root-project" in output
+    assert f"{run.INFO}   □ satellite-project" in output
 
 
-def test_print_project_start_reports_empty_projects(capsys):
+def test_print_execution_plan_marks_packages(tmp_path, capsys):
+    root_dir = tmp_path / "root"
+    root_dir.mkdir()
+    package_root = tmp_path / "pkg"
+    package_root.mkdir()
+    (package_root / "package.yaml").write_text("name: pkg\n")
+    plan = run.ExecutionPlan(
+        root=run.ProjectSelection(
+            root=root_dir,
+            selectors=[],
+            run_all=True,
+            kind="root",
+        ),
+        satellites=[
+            run.ProjectSelection(
+                root=package_root,
+                selectors=[],
+                run_all=True,
+                kind="satellite",
+            )
+        ],
+    )
+
+    run._print_execution_plan(plan, display_base=root_dir)
+
+    output = capsys.readouterr().out
+    assert "found 2 projects" in output
+    assert f"{run.INFO}   ■ root" in output
+    assert f"{run.INFO}   ○ pkg" in output
+
+
+def test_print_project_start_reports_empty_projects(tmp_path, capsys):
+    project_root = tmp_path / "satellite"
+    project_root.mkdir()
+    selection = run.ProjectSelection(
+        root=project_root,
+        selectors=[],
+        run_all=True,
+        kind="satellite",
+    )
+
     run._print_project_start(
-        relative_path="sat-project",
+        selection=selection,
+        display_base=tmp_path,
         queue_size=0,
         job_count=0,
         enabled_flags="",
+        verb="running",
     )
 
     output = capsys.readouterr().out
-    assert "running 0 tests" in output
-    assert "sat-project" in output
+    assert (
+        output
+        == f"{run.INFO} {run.BOLD}satellite{run.RESET_COLOR}: running 0 tests from satellite project at ./satellite\n"
+    )
+
+
+def test_summarize_harness_configuration_sets_update_verb() -> None:
+    job_count, enabled_flags, verb = run._summarize_harness_configuration(
+        jobs=1,
+        update=True,
+        coverage=False,
+        debug=False,
+        show_summary=False,
+        runner_summary=False,
+        fixture_summary=False,
+        passthrough=False,
+    )
+
+    assert job_count == 1
+    assert "update" not in enabled_flags
+    assert verb == "updating"
+
+
+def test_summarize_harness_configuration_sets_passthrough_verb() -> None:
+    job_count, enabled_flags, verb = run._summarize_harness_configuration(
+        jobs=2,
+        update=False,
+        coverage=False,
+        debug=False,
+        show_summary=False,
+        runner_summary=False,
+        fixture_summary=False,
+        passthrough=True,
+    )
+
+    assert job_count == 2
+    assert "passthrough" not in enabled_flags
+    assert verb == "showing"
+
+
+def test_print_project_start_uses_custom_verb(tmp_path, capsys):
+    project_root = tmp_path / "example"
+    project_root.mkdir()
+    selection = run.ProjectSelection(
+        root=project_root,
+        selectors=[],
+        run_all=True,
+        kind="root",
+    )
+
+    run._print_project_start(
+        selection=selection,
+        display_base=tmp_path,
+        queue_size=3,
+        job_count=0,
+        enabled_flags="",
+        verb="showing",
+    )
+
+    output = capsys.readouterr().out
+    assert (
+        output
+        == f"{run.INFO} {run.BOLD}example{run.RESET_COLOR}: showing 3 tests from root project at ./example\n"
+    )
 
 
 def test_execution_plan_single_project_summary(tmp_path, monkeypatch, capsys):
@@ -1041,9 +1308,7 @@ def test_execution_plan_single_project_summary(tmp_path, monkeypatch, capsys):
     run._print_execution_plan(plan, display_base=root)
 
     output = capsys.readouterr().out
-    assert "executing project" in output
-    assert "root:" not in output
-    assert "satellite:" not in output
+    assert output == ""
 
 
 def test_execution_plan_skips_inactive_root(capsys):
@@ -1068,9 +1333,7 @@ def test_execution_plan_skips_inactive_root(capsys):
     run._print_execution_plan(plan, display_base=root)
 
     output = capsys.readouterr().out
-    assert "executing project" in output
-    assert "satellite-project" in output
-    assert "root-project" not in output
+    assert output == ""
 
 
 def test_print_aggregate_totals(capsys):
